import UIKit

//**********************************************************************
//How to extract a JSON object into a Swift struct
//**********************************************************************

struct MyStruct_1:Decodable
{
    var name:String
    var id:Int
    var some_extra_param:String? = nil
}

let json_1 = """
            [
                {"name":"test 1", "id" : 1},
                {"name":"test 2", "id" : 2, "some_extra_param" : "param"}
            ]
           """.data(using: .utf8)!


let Arr = try? JSONDecoder().decode([MyStruct_1].self, from: json_1)

if let Arr = Arr
{
    Arr.forEach{print($0)}
}

//**********************************************************************


//**********************************************************************
//How to do a custom parsing
//**********************************************************************

struct MyStruct_2
{
    var name:String
    var id:Int
    var some_extra_param:String? = nil
    
    enum keys:String,CodingKey
    {
        case name
        case id
        case some_extra_param
    }
}

extension MyStruct_2:Decodable
{
    init(from decoder:Decoder) throws
    {
        let values = try decoder.container(keyedBy: keys.self)
        
        name = try values.decode(String.self, forKey: .name)
        some_extra_param = try? values.decode(String.self, forKey: .some_extra_param)
        let id_as_string = try values.decode(String.self, forKey: .id)

        id = Int(id_as_string)!
        
        if(some_extra_param == nil)
        {
            some_extra_param = "not_defined"
        }
    }
}

let json_2 = """
            [
                {"name":"test 1", "id" : "1"},
                {"name":"test 2", "id" : "2", "some_extra_param" : "param"}
            ]
           """.data(using: .utf8)!

let Arr2 = try? JSONDecoder().decode([MyStruct_2].self, from: json_2)

if let Arr = Arr2
{
    Arr.forEach{print($0)}
}
//**********************************************************************



//**********************************************************************
//How to do encode a struct to JSON
//**********************************************************************

struct MyStruct_3:Encodable //if your struct should be both encodable and codable, use Codable
{
    let name:String
    let id:Int
}

var Arr3 = [MyStruct_3]()
Arr3.append(MyStruct_3(name: "test 1", id: 1))
Arr3.append(MyStruct_3(name: "test 2", id: 1))

if let json_data = try? JSONEncoder().encode(Arr3)
{
    if let json_string = String(data:json_data, encoding: .utf8)
    {
        print(json_string)
    }
}
//**********************************************************************

